==> clojure-lsp-2022.09.01-15.27.31.tar.gz: clojure-lsp-2022.09.01-15.27.31/lib/src/clojure_lsp/feature/document_symbol.clj <==
(ns clojure-lsp.feature.document-symbol
  (:require
   [clojure-lsp.queries :as q]
   [clojure-lsp.shared :as shared]))

(set! *warn-on-reflection* true)

(defn element->symbol-kind [el]
  (cond
    (#{:namespace-usages :namespace-definitions} (:bucket el)) :namespace
    (or (:macro el)
        (:fixed-arities el)) :function
    (#{:var-definitions :var-usages} (:bucket el)) :variable
    :else :null))

(defn element->name [{elem-name :name :keys [dispatch-val-str]}]
  (cond-> (name elem-name)
    dispatch-val-str (str " " dispatch-val-str)))

(defn ^:private element->document-symbol [e]
  (shared/assoc-some
    {:name (element->name e)
     :kind (element->symbol-kind e)
     :range (shared/->scope-range e)
     :selection-range (shared/->range e)
     :tags (cond-> []
             (:deprecated e) (conj 1))}
    :detail (when (:private e)
              "private")))

(defn ^:private symbol-order [{:keys [selection-range]}]
  [(:line (:start selection-range)) (:character (:start selection-range))])

(defn document-symbols [db filename]
  (let [namespace-definition (q/find-namespace-definition-by-filename db filename)]
    [{:name (or (some-> namespace-definition :name name)
                filename)
      :kind (element->symbol-kind namespace-definition)
      :range shared/full-file-range
      :selection-range (if namespace-definition
                         (shared/->scope-range namespace-definition)
                         shared/full-file-range)
      :children (->> (concat (q/find-var-definitions db filename true)
                             (q/find-defmethods db filename))
                     (map element->document-symbol)
                     (sort-by symbol-order)
                     vec)}]))

==> clojure-lsp-2022.11.03-00.14.57.tar.gz: clojure-lsp-2022.11.03-00.14.57/lib/src/clojure_lsp/feature/document_symbol.clj <==
(ns clojure-lsp.feature.document-symbol
  (:require
   [clojure-lsp.queries :as q]
   [clojure-lsp.shared :as shared]))

(set! *warn-on-reflection* true)

(defn element->symbol-kind [el]
  (cond
    (#{:namespace-usages :namespace-definitions} (:bucket el)) :namespace
    (or (:macro el)
        (:fixed-arities el)) :function
    (#{:var-definitions :var-usages} (:bucket el)) :variable
    :else :null))

(defn element->name [{elem-name :name :keys [dispatch-val-str]}]
  (cond-> (name elem-name)
    dispatch-val-str (str " " dispatch-val-str)))

(defn ^:private element->document-symbol [e]
  (shared/assoc-some
    {:name (element->name e)
     :kind (element->symbol-kind e)
     :range (shared/->scope-range e)
     :selection-range (shared/->range e)
     :tags (cond-> []
             (:deprecated e) (conj 1))}
    :detail (when (:private e)
              "private")))

(defn ^:private symbol-order [{:keys [selection-range]}]
  [(:line (:start selection-range)) (:character (:start selection-range))])

(defn document-symbols [db uri]
  (let [namespace-definition (q/find-namespace-definition-by-uri db uri)]
    [{:name (or (some-> namespace-definition :name name)
                ;; TODO Consider using URI for display purposes, especially if
                ;; we support remote LSP connections
                (shared/uri->filename uri))
      :kind (element->symbol-kind namespace-definition)
      :range shared/full-file-range
      :selection-range (if namespace-definition
                         (shared/->scope-range namespace-definition)
                         shared/full-file-range)
      :children (->> (concat (q/find-var-definitions db uri true)
                             (q/find-defmethods db uri))
                     (map element->document-symbol)
                     (sort-by symbol-order)
                     vec)}]))
